# -*- coding: utf-8 -*-
import django
from django.contrib import admin
from django.utils import timezone, formats
from django.utils.safestring import mark_safe

if django.VERSION[0] < 3:
    from django.utils.translation import ungettext as ngettext
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import ngettext as ngettext
    from django.utils.translation import gettext_lazy as _

from publishable_model import settings as app_settings
from publishable_model.admin.admin_filters import IsPublishedFilter


class PublishableModelAdmin(admin.ModelAdmin):
    publication_field = "status"
    date_hierarchy = "publication_start"
    actions = ["make_published", "make_unpublished"]
    DISPLAY_PUBLICATION_START = False
    DISPLAY_PUBLICATION_END = False
    DISPLAY_PUBLICATION_DATES = False

    def get_list_filter(self, request):
        """Override get_list_filter to allow final user to filter on published or draft items."""
        list_filter = list(super(PublishableModelAdmin, self).get_list_filter(request))
        list_filter.append(IsPublishedFilter)
        return list_filter

    def get_list_display(self, request):
        """Override get_list_display to show publication fields in change_list.html."""
        list_display = list(
            super(PublishableModelAdmin, self).get_list_display(request)
        )
        if "display_actual_status" not in list_display:
            list_display.append("display_actual_status")
        if (
            self.DISPLAY_PUBLICATION_START
            and "display_publication_start" not in self.list_display
        ):
            list_display.append("display_publication_start")
        if (
            self.DISPLAY_PUBLICATION_END
            and "display_publication_end" not in self.list_display
        ):
            list_display.append("display_publication_end")
        if (
            self.DISPLAY_PUBLICATION_DATES
            and "display_publication_dates" not in self.list_display
        ):
            list_display.append("display_publication_dates")
        return list_display

    # ===========================================
    # List Display Fields
    # ===========================================
    def display_actual_status(self, obj):
        return obj.is_actual

    display_actual_status.short_description = _("is published")
    display_actual_status.boolean = True

    def display_publication_start(self, obj=None):
        return (
            formats.date_format(
                timezone.localtime(
                    obj.publication_start, timezone.get_current_timezone()
                ),
                "SHORT_DATETIME_FORMAT",
            )
            if obj.publication_start
            else ""
        )

    display_publication_start.short_description = _("Publication Start Date")
    display_publication_start.admin_order_field = "publication_start"

    def display_publication_end(self, obj=None):
        return (
            formats.date_format(
                timezone.localtime(
                    obj.publication_end, timezone.get_current_timezone()
                ),
                "SHORT_DATETIME_FORMAT",
            )
            if obj.publication_end
            else ""
        )

    display_publication_end.short_description = _("Publication End Date")
    display_publication_end.admin_order_field = "publication_end"

    @mark_safe
    def display_publication_dates(self, obj):
        dt = ""
        if obj and obj.id:
            if obj.publication_start:
                dt += "{0} {1}" "".format(
                    _("From"),
                    formats.date_format(
                        timezone.localtime(
                            obj.publication_start, timezone.get_current_timezone()
                        ),
                        "SHORT_DATETIME_FORMAT",
                    ),
                )
                if obj.publication_end:
                    dt += "<br>{0} {1}" "".format(
                        _("To"),
                        formats.date_format(
                            timezone.localtime(
                                obj.publication_end, timezone.get_current_timezone()
                            ),
                            "SHORT_DATETIME_FORMAT",
                        ),
                    )
        return mark_safe(dt)

    display_publication_dates.short_description = _("Publication Dates")
    display_publication_dates.admin_order_field = "publication_start"

    # ===========================================
    # Actions
    # ===========================================
    def make_published(self, request, queryset):
        rows_updated = queryset.update(
            status=app_settings.PUBLICATION_STATUS_PUBLISHED,
            publication_start=timezone.now(),
            publication_end=None,
        )
        self.message_user(
            request,
            ngettext(
                "%(count)d entries was published",
                "%(count)d entries where published",
                rows_updated,
            )
            % {"count": rows_updated},
        )

    make_published.short_description = _("Publish selected entries")

    def make_unpublished(self, request, queryset):
        rows_updated = queryset.update(
            status=app_settings.PUBLICATION_STATUS_DRAFT,
            publication_start=None,
            publication_end=None,
        )
        self.message_user(
            request,
            ngettext(
                "%(count)d entries was unpublished",
                "%(count)d entries where unpublished",
                rows_updated,
            )
            % {"count": rows_updated},
        )

    make_unpublished.short_description = _("Unpublish selected entries")



# -*- coding: utf-8 -*-
from django.db import models

from .querysets import PublishableQuerySet


class PublishableModelManager(models.Manager):
    use_for_related_fields = True

    def get_queryset(self):
        return PublishableQuerySet(self.model, using=self._db)


class PublishedModelManager(PublishableModelManager):

    def get_queryset(self):
        return super(PublishedModelManager, self).get_queryset().published_items()


class NotPublishedModelManager(PublishableModelManager):

    def get_queryset(self):
        return super(NotPublishedModelManager, self).get_queryset().not_published_items()


class DraftModelManager(PublishableModelManager):

    def get_queryset(self):
        return super(DraftModelManager, self).get_queryset().draft_items()



# -*- coding: utf-8 -*-
from django.db import models
from django.utils import timezone
from . import settings as app_settings


class PublishableQuerySet(models.QuerySet):

    def published_items(self):
        now = timezone.now()
        q_pub_start = models.Q(publication_start__lte=now)
        q_pub_end = (models.Q(publication_end__gte=now) | models.Q(publication_end__isnull=True))
        q_status = models.Q(status=app_settings.PUBLICATION_STATUS_PUBLISHED)
        return self.filter(q_pub_start & q_pub_end & q_status)

    def not_published_items(self):
        now = timezone.now()
        q_pub_start = models.Q(publication_start__lte=now)
        q_pub_end = (models.Q(publication_end__gte=now) | models.Q(publication_end__isnull=True))
        q_status = models.Q(status=app_settings.PUBLICATION_STATUS_PUBLISHED)
        return self.exclude(q_pub_start & q_pub_end & q_status)

    def draft_items(self):
        q_status = models.Q(status=app_settings.PUBLICATION_STATUS_DRAFT)
        return self.filter(q_status)



# -*- coding: utf-8 -*-
import django
from django.db import models
from django.utils import timezone

from . import managers as app_managers
from . import settings as app_settings

if django.VERSION[0] < 3:
    from django.utils.translation import ugettext as gettext
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import gettext as gettext
    from django.utils.translation import gettext_lazy as _


class PublishableModel(models.Model):
    objects = app_managers.PublishableModelManager()
    published_objects = app_managers.PublishedModelManager()
    not_published_objects = app_managers.NotPublishedModelManager()
    draft_objects = app_managers.DraftModelManager()

    PUBLICATION_STATUS_CHOICES = (
        (app_settings.PUBLICATION_STATUS_DRAFT, _("draft")),
        (app_settings.PUBLICATION_STATUS_PUBLISHED, _("published")),
    )

    status = models.CharField(
        _("status"),
        max_length=2,
        db_index=True,
        choices=PUBLICATION_STATUS_CHOICES,
        default=app_settings.PUBLICATION_STATUS_DRAFT,
    )
    publication_start = models.DateTimeField(
        _("publication start"), null=True, blank=True
    )
    publication_end = models.DateTimeField(_("publication end"), null=True, blank=True)

    class Meta:
        abstract = True
        ordering = [
            "-publication_start",
        ]
        get_latest_by = "publication_start"

    def get_is_actual(self):
        return all(
            [
                self.publication_start and self.publication_start < timezone.now(),
                self.publication_end is None or self.publication_end > timezone.now(),
                self.status == app_settings.PUBLICATION_STATUS_PUBLISHED,
            ]
        )

    is_actual = property(get_is_actual)

    def save(self, *args, **kwargs):
        if (
            self.status == app_settings.PUBLICATION_STATUS_PUBLISHED
            and not self.publication_start
        ):
            self.publication_start = timezone.now()
        super(PublishableModel, self).save(*args, **kwargs)

    def publish(self, commit=True, update_publication_start=False):
        if self.status != app_settings.PUBLICATION_STATUS_PUBLISHED:
            self.status = app_settings.PUBLICATION_STATUS_PUBLISHED
            if update_publication_start:  # and not self.publication_start:
                self.publication_start = timezone.now()
            if commit:
                self.save()

    def unpublish(
        self, commit=True, update_publication_start=False, update_publication_end=False
    ):
        if self.status != app_settings.PUBLICATION_STATUS_DRAFT:
            self.status = app_settings.PUBLICATION_STATUS_DRAFT
            if update_publication_start and self.publication_start:
                self.publication_start = None
            if update_publication_end and self.publication_end:
                self.publication_end = None
            if commit:
                self.save()



# -*- coding: utf-8 -*-
import django
from django.contrib import messages
from django.contrib.admin.filters import SimpleListFilter

if django.VERSION[0] < 3:
    from django.utils.translation import ungettext as ngettext
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import ngettext as ngettext
    from django.utils.translation import gettext_lazy as _


class SimpleBooleanListFilter(SimpleListFilter):
    title = ''  # _('Your Filter Label')
    parameter_name = ''  # 'your_filter_key'
    is_legacy = True
    LOOKUP_CHOICES = (
        ('1', _('Yes')),
        ('0', _('No')),
    )

    def lookups(self, request, model_admin):
        return self.LOOKUP_CHOICES

    def get_allowed_values(self):
        return dict(self.LOOKUP_CHOICES).keys()

    def get_true_queryset_values(self, queryset):
        raise NotImplementedError()

    def get_false_queryset_values(self, queryset):
        raise NotImplementedError()

    def queryset(self, request, queryset):
        _value = self.value()
        if _value in self.get_allowed_values():
            if bool(int(_value)):
                return self.get_true_queryset_values(queryset)
            else:
                return self.get_false_queryset_values(queryset)
        if self.value():
            messages.add_message(request, messages.WARNING,
                                 _("Impossible to filter with the argument '{0}' for the key '{1}'"
                                   "").format(self.value(), self.title))
        return queryset


class IsPublishedFilter(SimpleBooleanListFilter):
    title = _('Published')
    parameter_name = 'is_published'

    def get_true_queryset_values(self, queryset):
        return queryset.published_items()

    def get_false_queryset_values(self, queryset):
        return queryset.draft_items()
